// Decompiled by Jad v1.5.8g. Copyright 2001 Pavel Kouznetsov.
// Jad home page: http://www.kpdus.com/jad.html
// Decompiler options: packimports(3) 
// Source File Name:   ServerConfigurationParser.java

package flex.messaging.config;

import flex.messaging.util.LocaleUtils;
import java.util.*;
import org.w3c.dom.*;

// Referenced classes of package flex.messaging.config:
//            AbstractConfigurationParser, ConfigurationException, ClusterSettings, MessagingConfiguration, 
//            SecurityConstraint, ChannelSettings, ServiceSettings, AdapterSettings, 
//            DestinationSettings, LoggingSettings, TargetSettings, SystemSettings, 
//            FactorySettings, SecuritySettings, ServicesConfiguration, ConfigurationFileResolver

public abstract class ServerConfigurationParser extends AbstractConfigurationParser
{

    public ServerConfigurationParser()
    {
    }

    protected void parseTopLevelConfig(Document doc)
    {
        Node root = selectSingleNode(doc, "/services-config");
        if(root != null)
        {
            allowedChildElements(root, SERVICES_CONFIG_CHILDREN);
            securitySection(root);
            channelsSection(root);
            services(root);
            clusters(root);
            logging(root);
            debugging(root);
            system(root);
            factories(root);
        } else
        {
            ConfigurationException e = new ConfigurationException();
            e.setMessage(10103, new Object[] {
                "services-config"
            });
            throw e;
        }
    }

    private void clusters(Node root)
    {
        Node clusteringNode = selectSingleNode(root, "clusters");
        if(clusteringNode != null)
        {
            allowedAttributesOrElements(clusteringNode, CLUSTERING_CHILDREN);
            NodeList clusters = selectNodeList(clusteringNode, "cluster");
            for(int i = 0; i < clusters.getLength(); i++)
            {
                Node cluster = clusters.item(i);
                requiredAttributesOrElements(cluster, CLUSTER_DEFINITION_CHILDREN);
                String clusterName = getAttributeOrChildElement(cluster, "id");
                if(isValidID(clusterName))
                {
                    String propsFileName = getAttributeOrChildElement(cluster, "properties");
                    ClusterSettings clusterSettings = new ClusterSettings();
                    clusterSettings.setClusterName(clusterName);
                    clusterSettings.setPropsFileName(propsFileName);
                    ((MessagingConfiguration)config).addClusterSettings(clusterSettings);
                }
            }

        }
    }

    private void securitySection(Node root)
    {
        Node security = selectSingleNode(root, "security");
        if(security != null)
        {
            allowedChildElements(security, SECURITY_CHILDREN);
            NodeList list = selectNodeList(security, "security-constraint");
            for(int i = 0; i < list.getLength(); i++)
            {
                Node constraint = list.item(i);
                securityConstraint(constraint, false);
            }

            list = selectNodeList(security, "login-command");
            for(int i = 0; i < list.getLength(); i++)
            {
                Node login = list.item(i);
                requiredAttributesOrElements(login, LOGIN_COMMAND_CHILDREN);
                String server = getAttributeOrChildElement(login, "server");
                if(server.length() == 0)
                {
                    ConfigurationException e = new ConfigurationException();
                    e.setMessage(10105, new Object[] {
                        "server", "login-command"
                    });
                    throw e;
                }
                String loginClass = getAttributeOrChildElement(login, "class");
                if(loginClass.length() == 0)
                {
                    ConfigurationException e = new ConfigurationException();
                    e.setMessage(10105, new Object[] {
                        "class", "login-command"
                    });
                    throw e;
                }
                ((MessagingConfiguration)config).getSecuritySettings().addLoginCommand(server, loginClass);
            }

        }
    }

    private SecurityConstraint securityConstraint(Node constraint, boolean inline)
    {
        allowedAttributesOrElements(constraint, SECURITY_CONSTRAINT_DEFINITION_CHILDREN);
        String ref = getAttributeOrChildElement(constraint, "ref");
        SecurityConstraint sc;
        if(ref.length() > 0)
        {
            allowedAttributesOrElements(constraint, new String[] {
                "ref"
            });
            sc = ((MessagingConfiguration)config).getSecuritySettings().getConstraint(ref);
            if(sc == null)
            {
                ConfigurationException e = new ConfigurationException();
                e.setMessage(10109, new Object[] {
                    "security-constraint", ref
                });
                throw e;
            }
        } else
        {
            String id = getAttributeOrChildElement(constraint, "id");
            if(inline)
                sc = new SecurityConstraint("");
            else
            if(isValidID(id))
            {
                sc = new SecurityConstraint(id);
                ((MessagingConfiguration)config).getSecuritySettings().addConstraint(sc);
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10110, new Object[] {
                    "security-constraint", id
                });
                ex.setDetails(10110);
                throw ex;
            }
            String method = getAttributeOrChildElement(constraint, "auth-method");
            sc.setMethod(method);
            Node rolesNode = selectSingleNode(constraint, "roles");
            if(rolesNode != null)
            {
                allowedChildElements(rolesNode, ROLES_CHILDREN);
                NodeList roles = selectNodeList(rolesNode, "role");
                for(int r = 0; r < roles.getLength(); r++)
                {
                    Node roleNode = roles.item(r);
                    String role = evaluateExpression(roleNode, ".").toString().trim();
                    if(role.length() > 0)
                        sc.addRole(role);
                }

            }
        }
        return sc;
    }

    private void channelsSection(Node root)
    {
        Node channelsNode = selectSingleNode(root, "channels");
        if(channelsNode != null)
        {
            allowedAttributesOrElements(channelsNode, CHANNELS_CHILDREN);
            NodeList channels = selectNodeList(channelsNode, "channel-definition");
            for(int i = 0; i < channels.getLength(); i++)
            {
                Node channel = channels.item(i);
                channelDefinition(channel);
            }

        }
    }

    private void channelDefinition(Node channel)
    {
        requiredAttributesOrElements(channel, CHANNEL_DEFINITION_REQ_CHILDREN);
        allowedAttributesOrElements(channel, CHANNEL_DEFINITION_CHILDREN);
        String id = getAttributeOrChildElement(channel, "id");
        if(isValidID(id))
        {
            ChannelSettings channelSettings = new ChannelSettings(id);
            channelSettings.setSourceFile(getSourceFileOf(channel));
            Node endpoint = selectSingleNode(channel, "endpoint");
            if(endpoint != null)
            {
                allowedAttributesOrElements(endpoint, ENDPOINT_REQ_CHILDREN);
                String type = getAttributeOrChildElement(endpoint, "class");
                channelSettings.setEndpointType(type);
                String uri = getAttributeOrChildElement(endpoint, "uri");
                channelSettings.setUri(uri);
                config.addChannelSettings(id, channelSettings);
            }
            NodeList properties = selectNodeList(channel, "properties/*");
            if(properties.getLength() > 0)
            {
                ConfigMap map = properties(properties);
                channelSettings.addProperties(map);
            }
            String ref = evaluateExpression(channel, "@security-constraint").toString().trim();
            if(ref.length() > 0)
            {
                SecurityConstraint sc = ((MessagingConfiguration)config).getSecuritySettings().getConstraint(ref);
                if(sc != null)
                {
                    channelSettings.setConstraint(sc);
                } else
                {
                    ConfigurationException ex = new ConfigurationException();
                    ex.setMessage(10132, new Object[] {
                        "security-constraint", ref, id
                    });
                    throw ex;
                }
            } else
            {
                Node security = selectSingleNode(channel, "security");
                if(security != null)
                {
                    allowedChildElements(security, EMBEDDED_SECURITY_CHILDREN);
                    Node constraint = selectSingleNode(security, "security-constraint");
                    if(constraint != null)
                    {
                        SecurityConstraint sc = securityConstraint(constraint, true);
                        channelSettings.setConstraint(sc);
                    }
                }
            }
        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10110, new Object[] {
                "channel-definition", id
            });
            ex.setDetails(10110);
            throw ex;
        }
    }

    private void services(Node root)
    {
        Node servicesNode = selectSingleNode(root, "services");
        if(servicesNode != null)
        {
            allowedChildElements(servicesNode, SERVICES_CHILDREN);
            NodeList services = selectNodeList(servicesNode, "service-include");
            for(int i = 0; i < services.getLength(); i++)
            {
                Node service = services.item(i);
                serviceInclude(service);
            }

            services = selectNodeList(servicesNode, "service");
            for(int i = 0; i < services.getLength(); i++)
            {
                Node service = services.item(i);
                service(service);
            }

        }
    }

    private void serviceInclude(Node serviceInclude)
    {
        requiredAttributesOrElements(serviceInclude, SERVICE_INCLUDE_CHILDREN);
        String src = getAttributeOrChildElement(serviceInclude, "file-path");
        if(src.length() > 0)
        {
            Document doc = loadDocument(src, fileResolver.getIncludedFile(src));
            doc.getDocumentElement().normalize();
            Node service = selectSingleNode(doc, "/service");
            if(service != null)
            {
                service(service);
                fileResolver.popIncludedFile();
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10112, new Object[] {
                    "service"
                });
                throw ex;
            }
        }
    }

    private void service(Node service)
    {
        requiredAttributesOrElements(service, SERVICE_REQ_CHILDREN);
        allowedAttributesOrElements(service, SERVICE_CHILDREN);
        String id = getAttributeOrChildElement(service, "id");
        if(isValidID(id))
        {
            ServiceSettings serviceSettings = config.getServiceSettings(id);
            if(serviceSettings == null)
            {
                serviceSettings = new ServiceSettings(id);
                serviceSettings.setSourceFile(getSourceFileOf(service));
                config.addServiceSettings(serviceSettings);
            } else
            {
                ConfigurationException e = new ConfigurationException();
                e.setMessage(10113, new Object[] {
                    id
                });
                throw e;
            }
            String className = getAttributeOrChildElement(service, "class");
            if(className.length() > 0)
            {
                serviceSettings.setClassName(className);
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10114, new Object[] {
                    "service", id
                });
                throw ex;
            }
            String messageTypesList = getAttributeOrChildElement(service, "messageTypes");
            if(messageTypesList.length() > 0)
            {
                StringTokenizer tokenizer = new StringTokenizer(messageTypesList, ",;:");
                do
                {
                    if(!tokenizer.hasMoreTokens())
                        break;
                    String messageType = tokenizer.nextToken().trim();
                    if(messageType.length() > 0)
                        serviceSettings.addMessageType(messageType);
                } while(true);
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10115, new Object[] {
                    "service", id
                });
                throw ex;
            }
            NodeList properties = selectNodeList(service, "properties/*");
            if(properties.getLength() > 0)
            {
                ConfigMap map = properties(properties);
                serviceSettings.addProperties(map);
            }
            Node defaultChannels = selectSingleNode(service, "default-channels");
            if(defaultChannels != null)
            {
                allowedChildElements(defaultChannels, DEFAULT_CHANNELS_CHILDREN);
                NodeList channels = selectNodeList(defaultChannels, "channel");
                for(int c = 0; c < channels.getLength(); c++)
                {
                    Node chan = channels.item(c);
                    defaultChannel(chan, serviceSettings);
                }

            }
            Node adapters = selectSingleNode(service, "adapters");
            if(adapters != null)
            {
                allowedChildElements(adapters, ADAPTERS_CHILDREN);
                NodeList serverAdapters = selectNodeList(adapters, "adapter-definition");
                for(int a = 0; a < serverAdapters.getLength(); a++)
                {
                    Node adapter = serverAdapters.item(a);
                    adapterDefinition(adapter, serviceSettings);
                }

            }
            NodeList list = selectNodeList(service, "destination");
            for(int i = 0; i < list.getLength(); i++)
            {
                Node dest = list.item(i);
                destination(dest, serviceSettings);
            }

            list = selectNodeList(service, "destination-include");
            for(int i = 0; i < list.getLength(); i++)
            {
                Node dest = list.item(i);
                destinationInclude(dest, serviceSettings);
            }

        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10110, new Object[] {
                "service", id
            });
            throw ex;
        }
    }

    private void defaultChannel(Node chan, ServiceSettings serviceSettings)
    {
        String ref = getAttributeOrChildElement(chan, "ref");
        if(ref.length() > 0)
        {
            ChannelSettings channel = config.getChannelSettings(ref);
            if(channel != null)
            {
                serviceSettings.addDefaultChannel(channel);
            } else
            {
                ConfigurationException e = new ConfigurationException();
                e.setMessage(10109, new Object[] {
                    "channel", ref
                });
                throw e;
            }
        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10116, new Object[] {
                serviceSettings.getId()
            });
            throw ex;
        }
    }

    private void adapterDefinition(Node adapter, ServiceSettings serviceSettings)
    {
        requiredAttributesOrElements(adapter, ADAPTER_DEFINITION_REQ_CHILDREN);
        allowedChildElements(adapter, ADAPTER_DEFINITION_CHILDREN);
        String serviceId = serviceSettings.getId();
        String id = getAttributeOrChildElement(adapter, "id");
        if(isValidID(id))
        {
            AdapterSettings adapterSettings = new AdapterSettings(id);
            adapterSettings.setSourceFile(getSourceFileOf(adapter));
            String className = getAttributeOrChildElement(adapter, "class");
            if(className.length() > 0)
            {
                adapterSettings.setClassName(className);
                boolean isDefault = Boolean.valueOf(getAttributeOrChildElement(adapter, "default")).booleanValue();
                if(isDefault)
                {
                    adapterSettings.setDefault(isDefault);
                    AdapterSettings defaultAdapter = serviceSettings.getDefaultAdapter();
                    if(defaultAdapter != null)
                    {
                        ConfigurationException ex = new ConfigurationException();
                        ex.setMessage(10117, new Object[] {
                            id, serviceId, defaultAdapter.getId()
                        });
                        throw ex;
                    }
                }
                serviceSettings.addAdapterSettings(adapterSettings);
                NodeList properties = selectNodeList(adapter, "properties/*");
                if(properties.getLength() > 0)
                {
                    ConfigMap map = properties(properties);
                    adapterSettings.addProperties(map);
                }
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10114, new Object[] {
                    "adapter-definition", id
                });
                throw ex;
            }
        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10119, new Object[] {
                "adapter-definition", id, serviceId
            });
            throw ex;
        }
    }

    private void destinationInclude(Node destInclude, ServiceSettings serviceSettings)
    {
        requiredAttributesOrElements(destInclude, DESTINATION_INCLUDE_CHILDREN);
        String src = getAttributeOrChildElement(destInclude, "file-path");
        if(src.length() > 0)
        {
            Document doc = loadDocument(src, fileResolver.getIncludedFile(src));
            doc.getDocumentElement().normalize();
            Node dest = selectSingleNode(doc, "/destination");
            if(dest != null)
            {
                destination(dest, serviceSettings);
                fileResolver.popIncludedFile();
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10118, new Object[] {
                    "destination"
                });
                throw ex;
            }
        }
    }

    private void destination(Node dest, ServiceSettings serviceSettings)
    {
        requiredAttributesOrElements(dest, DESTINATION_REQ_CHILDREN);
        allowedAttributes(dest, DESTINATION_ATTR);
        allowedChildElements(dest, DESTINATION_CHILDREN);
        String serviceId = serviceSettings.getId();
        String id = getAttributeOrChildElement(dest, "id");
        DestinationSettings destinationSettings;
        if(isValidID(id))
        {
            destinationSettings = (DestinationSettings)serviceSettings.getDestinationSettings().get(id);
            if(destinationSettings != null)
            {
                ConfigurationException e = new ConfigurationException();
                e.setMessage(10122, new Object[] {
                    id, serviceId
                });
                throw e;
            }
            destinationSettings = new DestinationSettings(id);
            destinationSettings.setSourceFile(getSourceFileOf(dest));
            serviceSettings.addDestinationSettings(destinationSettings);
        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10119, new Object[] {
                "destination", id, serviceId
            });
            throw ex;
        }
        NodeList properties = selectNodeList(dest, "properties/*");
        if(properties.getLength() > 0)
        {
            ConfigMap map = properties(properties);
            destinationSettings.addProperties(map);
        }
        destinationChannels(dest, destinationSettings, serviceSettings);
        destinationSecurity(dest, destinationSettings);
        destinationAdapter(dest, destinationSettings, serviceSettings);
    }

    private void destinationChannels(Node dest, DestinationSettings destinationSettings, ServiceSettings serviceSettings)
    {
        String destId = destinationSettings.getId();
        String channelsList = evaluateExpression(dest, "@channels").toString().trim();
        if(channelsList.length() > 0)
        {
            for(StringTokenizer st = new StringTokenizer(channelsList, ",;:"); st.hasMoreTokens();)
            {
                String ref = st.nextToken();
                ChannelSettings channel = config.getChannelSettings(ref);
                if(channel != null)
                {
                    destinationSettings.addChannelSettings(channel);
                } else
                {
                    ConfigurationException ex = new ConfigurationException();
                    ex.setMessage(10120, new Object[] {
                        "channel", ref, destId
                    });
                    throw ex;
                }
            }

        } else
        {
            Node channelsNode = selectSingleNode(dest, "channels");
            if(channelsNode != null)
            {
                allowedChildElements(channelsNode, DESTINATION_CHANNELS_CHILDREN);
                NodeList channels = selectNodeList(channelsNode, "channel");
                for(int c = 0; c < channels.getLength(); c++)
                {
                    Node chan = channels.item(c);
                    requiredAttributesOrElements(chan, DESTINATION_CHANNEL_REQ_CHILDREN);
                    String ref = getAttributeOrChildElement(chan, "ref");
                    if(ref.length() > 0)
                    {
                        ChannelSettings channel = config.getChannelSettings(ref);
                        if(channel != null)
                        {
                            destinationSettings.addChannelSettings(channel);
                        } else
                        {
                            ConfigurationException ex = new ConfigurationException();
                            ex.setMessage(10120, new Object[] {
                                "channel", ref, destId
                            });
                            throw ex;
                        }
                    } else
                    {
                        ConfigurationException ex = new ConfigurationException();
                        ex.setMessage(10121, new Object[] {
                            "channel", ref, destId
                        });
                        throw ex;
                    }
                }

            } else
            {
                List defaultChannels = serviceSettings.getDefaultChannels();
                ChannelSettings channel;
                for(Iterator it = defaultChannels.iterator(); it.hasNext(); destinationSettings.addChannelSettings(channel))
                    channel = (ChannelSettings)it.next();

            }
        }
        if(destinationSettings.getChannelSettings().size() <= 0)
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10123, new Object[] {
                destId
            });
            throw ex;
        } else
        {
            return;
        }
    }

    private void destinationSecurity(Node dest, DestinationSettings destinationSettings)
    {
        String destId = destinationSettings.getId();
        String ref = evaluateExpression(dest, "@security-constraint").toString().trim();
        if(ref.length() > 0)
        {
            SecurityConstraint sc = ((MessagingConfiguration)config).getSecuritySettings().getConstraint(ref);
            if(sc != null)
            {
                destinationSettings.setConstraint(sc);
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10120, new Object[] {
                    "security-constraint", ref, destId
                });
                throw ex;
            }
        } else
        {
            Node security = selectSingleNode(dest, "security");
            if(security != null)
            {
                allowedChildElements(security, EMBEDDED_SECURITY_CHILDREN);
                Node constraint = selectSingleNode(security, "security-constraint");
                if(constraint != null)
                {
                    SecurityConstraint sc = securityConstraint(constraint, true);
                    destinationSettings.setConstraint(sc);
                }
            }
        }
    }

    private void destinationAdapter(Node dest, DestinationSettings destinationSettings, ServiceSettings serviceSettings)
    {
        String destId = destinationSettings.getId();
        String ref = evaluateExpression(dest, "@adapter").toString().trim();
        if(ref.length() > 0)
        {
            adapterReference(ref, destinationSettings, serviceSettings);
        } else
        {
            Node adapter = selectSingleNode(dest, "adapter");
            if(adapter != null)
            {
                allowedAttributesOrElements(adapter, DESTINATION_ADAPTER_CHILDREN);
                ref = getAttributeOrChildElement(adapter, "ref");
                adapterReference(ref, destinationSettings, serviceSettings);
            } else
            {
                AdapterSettings adapterSettings = serviceSettings.getDefaultAdapter();
                if(adapterSettings != null)
                    destinationSettings.setAdapterSettings(adapterSettings);
            }
        }
        if(destinationSettings.getAdapterSettings() == null)
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10127, new Object[] {
                destId
            });
            throw ex;
        } else
        {
            return;
        }
    }

    private void adapterReference(String ref, DestinationSettings destinationSettings, ServiceSettings serviceSettings)
    {
        String destId = destinationSettings.getId();
        if(ref.length() > 0)
        {
            AdapterSettings adapterSettings = serviceSettings.getAdapterSettings(ref);
            if(adapterSettings != null)
            {
                destinationSettings.setAdapterSettings(adapterSettings);
            } else
            {
                ConfigurationException ex = new ConfigurationException();
                ex.setMessage(10120, new Object[] {
                    "adapter", ref, destId
                });
                throw ex;
            }
        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10121, new Object[] {
                "adapter", ref, destId
            });
            throw ex;
        }
    }

    private void logging(Node root)
    {
        Node logging = selectSingleNode(root, "logging");
        if(logging != null)
        {
            allowedAttributesOrElements(logging, LOGGING_CHILDREN);
            LoggingSettings settings = new LoggingSettings();
            NodeList properties = selectNodeList(logging, "properties/*");
            if(properties.getLength() > 0)
            {
                ConfigMap map = properties(properties);
                settings.addProperties(map);
            }
            NodeList targets = selectNodeList(logging, "target");
            for(int i = 0; i < targets.getLength(); i++)
            {
                Node targetNode = targets.item(i);
                requiredAttributesOrElements(targetNode, TARGET_REQ_CHILDREN);
                allowedAttributesOrElements(targetNode, TARGET_CHILDREN);
                String className = getAttributeOrChildElement(targetNode, "class");
                if(className.length() <= 0)
                    continue;
                TargetSettings targetSettings = new TargetSettings(className);
                String targetLevel = getAttributeOrChildElement(targetNode, "level");
                if(targetLevel.length() > 0)
                    targetSettings.setLevel(targetLevel);
                Node filtersNode = selectSingleNode(targetNode, "filters");
                if(filtersNode != null)
                {
                    allowedChildElements(filtersNode, FILTERS_CHILDREN);
                    NodeList filters = selectNodeList(filtersNode, "pattern");
                    for(int f = 0; f < filters.getLength(); f++)
                    {
                        Node pattern = filters.item(f);
                        String filter = evaluateExpression(pattern, ".").toString().trim();
                        targetSettings.addFilter(filter);
                    }

                }
                properties = selectNodeList(targetNode, "properties/*");
                if(properties.getLength() > 0)
                {
                    ConfigMap map = properties(properties);
                    targetSettings.addProperties(map);
                }
                settings.addTarget(targetSettings);
            }

            config.setLoggingSettings(settings);
        }
    }

    private void system(Node root)
    {
        Node system = selectSingleNode(root, "system");
        if(system != null)
        {
            allowedAttributesOrElements(system, SYSTEM_CHILDREN);
            SystemSettings settings = new SystemSettings();
            Node localeNode = selectSingleNode(system, "locale");
            if(localeNode != null)
            {
                allowedAttributesOrElements(localeNode, LOCALE_CHILDREN);
                String defaultLocaleString = getAttributeOrChildElement(localeNode, "default-locale");
                java.util.Locale defaultLocale;
                if(defaultLocaleString.length() > 0)
                    defaultLocale = LocaleUtils.buildLocale(defaultLocaleString);
                else
                    defaultLocale = LocaleUtils.buildLocale(null);
                settings.setDefaultLocale(defaultLocale);
            }
            Node redeployNode = selectSingleNode(system, "redeploy");
            if(redeployNode != null)
            {
                allowedAttributesOrElements(redeployNode, REDEPLOY_CHILDREN);
                String enabled = getAttributeOrChildElement(redeployNode, "enabled");
                settings.setRedeployEnabled(enabled);
                String interval = getAttributeOrChildElement(redeployNode, "watch-interval");
                if(interval.length() > 0)
                    settings.setWatchInterval(interval);
                else
                    settings.setWatchInterval("20");
                NodeList watches = selectNodeList(redeployNode, "watch-file");
                for(int i = 0; i < watches.getLength(); i++)
                {
                    Node watchNode = watches.item(i);
                    String watch = evaluateExpression(watchNode, ".").toString().trim();
                    if(watch.length() > 0)
                        settings.addWatchFile(watch);
                }

                NodeList touches = selectNodeList(redeployNode, "touch-file");
                for(int i = 0; i < touches.getLength(); i++)
                {
                    Node touchNode = touches.item(i);
                    String touch = evaluateExpression(touchNode, ".").toString().trim();
                    if(touch.length() > 0)
                        settings.addTouchFile(touch);
                }

            }
            ((MessagingConfiguration)config).setSystemSettings(settings);
        }
    }

    private void factories(Node root)
    {
        Node factories = selectSingleNode(root, "factories");
        if(factories != null)
        {
            allowedAttributesOrElements(factories, FACTORIES_CHILDREN);
            NodeList factoryList = selectNodeList(factories, "factory");
            for(int i = 0; i < factoryList.getLength(); i++)
            {
                Node factory = factoryList.item(i);
                factory(factory);
            }

        }
    }

    private void factory(Node factory)
    {
        requiredAttributesOrElements(factory, FACTORY_REQ_CHILDREN);
        String id = getAttributeOrChildElement(factory, "id");
        String className = getAttributeOrChildElement(factory, "class");
        if(isValidID(id))
        {
            FactorySettings factorySettings = new FactorySettings(id, className);
            NodeList properties = selectNodeList(factory, "properties/*");
            if(properties.getLength() > 0)
            {
                ConfigMap map = properties(properties);
                factorySettings.addProperties(map);
            }
            ((MessagingConfiguration)config).addFactorySettings(id, factorySettings);
        } else
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10110, new Object[] {
                "factory", id
            });
            ex.setDetails(10110);
            throw ex;
        }
    }

    private void debugging(Node root)
    {
        Node debugging = selectSingleNode(root, "debugging");
        if(debugging == null);
    }
}
