// Decompiled by Jad v1.5.8g. Copyright 2001 Pavel Kouznetsov.
// Jad home page: http://www.kpdus.com/jad.html
// Decompiler options: packimports(3) 
// Source File Name:   AmfxInput.java

package flex.messaging.io.amfx;

import flex.messaging.MessageException;
import flex.messaging.io.*;
import flex.messaging.io.amf.*;
import flex.messaging.io.amf.serializers.CustomDeserializer;
import flex.messaging.io.amf.serializers.XMLDeserializer;
import flex.messaging.io.amfx.serializers.ByteArrayDeserializer;
import java.io.*;
import java.lang.reflect.Array;
import java.util.*;
import org.xml.sax.Attributes;

public class AmfxInput
{
    private class TraitsContext
    {

        private void add(String trait)
        {
            trait = trait.trim();
            if(trait.length() <= 0)
                throw new MessageException("Traits cannot be the empty string.");
            char c = trait.charAt(0);
            if(!Character.isLetterOrDigit(c) && c != '_')
            {
                throw new MessageException("Invalid trait name: " + trait + ". Object property names must start with a letter, a digit or the underscore '_' character.");
            } else
            {
                traits.add(trait);
                return;
            }
        }

        private String next()
        {
            String trait = (String)traits.get(counter);
            counter++;
            return trait;
        }

        private List traits;
        private int counter;



        private TraitsContext(List traits)
        {
            super();
            this.traits = traits;
        }

    }


    public AmfxInput(SerializationContext context)
    {
        beanproxy = new BeanProxy();
        this.context = context;
        configDefaults();
        objectStack = new Stack();
        arrayPropertyStack = new Stack();
        strictArrayIndexStack = new Stack();
        ecmaArrayIndexStack = new Stack();
        traitsStack = new Stack();
        text = new StringBuffer(32);
    }

    private void configDefaults()
    {
        xmlDeserializer = context.getCustomDeserializer("xml");
        byteArrayDeserializer = context.getCustomDeserializer("byte-array");
        if(xmlDeserializer == null)
            xmlDeserializer = DEFAULT_XML_DESERIALIZER;
        if(byteArrayDeserializer == null)
            byteArrayDeserializer = DEFAULT_BYTE_ARRAY_DESERIALIZER;
    }

    public void reset()
    {
        stringTable.clear();
        objectTable.clear();
        traitsTable.clear();
        objectStack.clear();
        arrayPropertyStack.clear();
        traitsStack.clear();
        currentBody = null;
        currentHeader = null;
        TypeMarshallingContext marshallingContext = TypeMarshallingContext.getTypeMarshallingContext();
        marshallingContext.reset();
    }

    public void setDebugTrace(AmfTrace trace)
    {
        this.trace = trace;
        isDebug = this.trace != null;
    }

    public void setActionMessage(ActionMessage msg)
    {
        message = msg;
    }

    public Object readObject()
        throws IOException
    {
        return null;
    }

    public void text(String s)
    {
        text.append(s);
    }

    public void start_amfx(Attributes attributes)
    {
        String ver = attributes.getValue("ver");
        int version = 3;
        if(ver != null)
            try
            {
                version = Integer.parseInt(ver);
            }
            catch(NumberFormatException ex)
            {
                throw new MessageException("Unknown version: " + ver);
            }
        if(isDebug)
            trace.version(version);
        message.setVersion(version);
    }

    public void end_amfx()
    {
    }

    public void start_header(Attributes attributes)
    {
        if(currentHeader != null || currentBody != null)
            throw new MessageException("Unexpected header tag.");
        currentHeader = new MessageHeader();
        String name = attributes.getValue("name");
        currentHeader.setName(name);
        String mu = attributes.getValue("mustUnderstand");
        boolean mustUnderstand = false;
        if(mu != null)
        {
            mustUnderstand = Boolean.valueOf(mu).booleanValue();
            currentHeader.setMustUnderstand(mustUnderstand);
        }
        if(isDebug)
            trace.startHeader(name, mustUnderstand, message.getHeaderCount());
    }

    public void end_header()
    {
        message.addHeader(currentHeader);
        currentHeader = null;
        if(isDebug)
            trace.endHeader();
    }

    public void start_body(Attributes attributes)
    {
        if(currentBody != null || currentHeader != null)
            throw new MessageException("Unexpected body tag.");
        currentBody = new MessageBody();
        String targetURI = attributes.getValue("targetURI");
        currentBody.setTargetURI(targetURI);
        String responseURI = attributes.getValue("responseURI");
        currentBody.setResponseURI(responseURI);
        if(isDebug)
            trace.startMessage(targetURI, responseURI, message.getBodyCount());
    }

    public void end_body()
    {
        message.addBody(currentBody);
        currentBody = null;
        if(isDebug)
            trace.endMessage();
    }

    public void start_array(Attributes attributes)
    {
        int length = 10;
        String len = attributes.getValue("length");
        if(len != null)
            try
            {
                len = len.trim();
                length = Integer.parseInt(len);
                if(length < 0)
                    throw new NumberFormatException();
            }
            catch(NumberFormatException ex)
            {
                throw new MessageException("Invalid array length: " + len);
            }
        String ecma = attributes.getValue("ecma");
        boolean isECMA = "true".equalsIgnoreCase(ecma);
        Object array;
        if(isECMA)
            array = new HashMap();
        else
        if(context.legacyCollection)
            array = new ArrayList(length);
        else
            array = ((Object) (new Object[length]));
        setValue(array);
        ecmaArrayIndexStack.push(new int[] {
            0
        });
        strictArrayIndexStack.push(new int[] {
            0
        });
        objectTable.add(array);
        objectStack.push(array);
        if(isECMA)
        {
            if(isDebug)
                trace.startECMAArray(objectTable.size() - 1);
        } else
        if(isDebug)
            trace.startAMFArray(objectTable.size() - 1);
    }

    public void end_array()
    {
        try
        {
            objectStack.pop();
            ecmaArrayIndexStack.pop();
            strictArrayIndexStack.pop();
        }
        catch(EmptyStackException ex)
        {
            throw new MessageException("Unexpected end of array");
        }
        if(isDebug)
            trace.endAMFArray();
    }

    public void start_bytearray(Attributes attributes)
    {
        text.delete(0, text.length());
    }

    public void end_bytearray()
    {
        String bs = text.toString().trim();
        Object value = byteArrayDeserializer.createObject(null, bs);
        setValue(value);
        if(isDebug)
            trace.startByteArray(objectTable.size() - 1, bs.length());
    }

    public void start_date(Attributes attributes)
    {
        text.delete(0, text.length());
    }

    public void end_date()
    {
        String d = text.toString().trim();
        try
        {
            long l = Long.parseLong(d);
            Date date = new Date(l);
            setValue(date);
            objectTable.add(date);
            if(isDebug)
                trace.write(date);
        }
        catch(NumberFormatException ex)
        {
            throw new MessageException("Invalid date: " + d);
        }
    }

    public void start_double(Attributes attributes)
    {
        text.delete(0, text.length());
    }

    public void end_double()
    {
        String ds = text.toString().trim();
        try
        {
            Double d = Double.valueOf(ds);
            setValue(d);
            if(isDebug)
                trace.write(d.doubleValue());
        }
        catch(NumberFormatException ex)
        {
            throw new MessageException("Invalid double: " + ds);
        }
    }

    public void start_false(Attributes attributes)
    {
        setValue(Boolean.FALSE);
        if(isDebug)
            trace.write(false);
    }

    public void end_false()
    {
    }

    public void start_item(Attributes attributes)
    {
        String name = attributes.getValue("name");
        if(name != null)
        {
            name = name.trim();
            if(name.length() <= 0)
                throw new MessageException("Array item names cannot be the empty string.");
            char c = name.charAt(0);
            if(!Character.isLetterOrDigit(c) && c != '_')
                throw new MessageException("Invalid item name: " + name + ". Array item names must start with a letter, a digit or the underscore '_' character.");
        } else
        {
            throw new MessageException("Array item must have a name attribute.");
        }
        Object o = objectStack.peek();
        if(!(o instanceof Map))
        {
            throw new MessageException("Unexpected array item name: " + name + ". Please set the ecma attribute to 'true'.");
        } else
        {
            arrayPropertyStack.push(name);
            return;
        }
    }

    public void end_item()
    {
        arrayPropertyStack.pop();
    }

    public void start_int(Attributes attributes)
    {
        text.delete(0, text.length());
    }

    public void end_int()
    {
        String is = text.toString().trim();
        try
        {
            Integer i = Integer.valueOf(is);
            setValue(i);
            if(isDebug)
                trace.write(i.intValue());
        }
        catch(NumberFormatException ex)
        {
            throw new MessageException("Invalid int: " + is);
        }
    }

    public void start_null(Attributes attributes)
    {
        setValue(null);
        if(isDebug)
            trace.writeNull();
    }

    public void end_null()
    {
    }

    public void start_object(Attributes attributes)
    {
        String type = attributes.getValue("type");
        if(type != null)
            type = type.trim();
        Object obj;
        if(type != null && type.length() > 0)
        {
            obj = beanproxy.createInstance(type);
            setValue(obj);
        } else
        {
            obj = new ASObject(type);
        }
        objectStack.push(obj);
        objectTable.add(obj);
        if(isDebug)
            trace.startAMFObject(type, objectTable.size() - 1);
    }

    public void end_object()
    {
        if(!traitsStack.empty())
            traitsStack.pop();
        if(!objectStack.empty())
        {
            Object obj = objectStack.pop();
            if(obj instanceof ASObject)
                setValue(obj);
        } else
        {
            throw new MessageException("Unexpected end of object.");
        }
        if(isDebug)
            trace.endAMFObject();
    }

    public void start_ref(Attributes attributes)
    {
        String id = attributes.getValue("id");
        if(id != null)
            try
            {
                int i = Integer.parseInt(id);
                Object o = objectTable.get(i);
                setValue(o);
                if(isDebug)
                    trace.writeRef(i);
            }
            catch(NumberFormatException ex)
            {
                throw new MessageException("Invalid object reference: " + id);
            }
            catch(IndexOutOfBoundsException ex)
            {
                throw new MessageException("Unknown object reference: " + id);
            }
        else
            throw new MessageException("Unknown object reference: " + id);
    }

    public void end_ref()
    {
    }

    public void start_string(Attributes attributes)
    {
        String id = attributes.getValue("id");
        if(id != null)
        {
            isStringReference = true;
            try
            {
                int i = Integer.parseInt(id);
                String s = (String)stringTable.get(i);
                if(isTraitProperty)
                {
                    TraitsContext traitsContext = (TraitsContext)traitsStack.peek();
                    traitsContext.add(s);
                } else
                {
                    setValue(s);
                }
            }
            catch(NumberFormatException ex)
            {
                throw new MessageException("Invalid string reference: " + id);
            }
            catch(IndexOutOfBoundsException ex)
            {
                throw new MessageException("Unknown string reference: " + id);
            }
        } else
        {
            text.delete(0, text.length());
            isStringReference = false;
        }
    }

    public void end_string()
    {
        if(!isStringReference)
        {
            String s = text.toString();
            if(s.length() > 0)
            {
                if(!isTraitProperty)
                    s = unescapeCloseCDATA(s);
                stringTable.add(s);
            }
            if(isTraitProperty)
            {
                TraitsContext traitsContext = (TraitsContext)traitsStack.peek();
                traitsContext.add(s);
            } else
            {
                setValue(s);
                if(isDebug)
                    trace.writeString(s);
            }
        }
    }

    public void start_traits(Attributes attributes)
    {
        if(!objectStack.empty())
        {
            List traitsList = new ArrayList();
            TraitsContext traitsContext = new TraitsContext(traitsList);
            traitsStack.push(traitsContext);
            String id = attributes.getValue("id");
            if(id != null)
            {
                try
                {
                    int i = Integer.parseInt(id);
                    List l = (List)traitsTable.get(i);
                    String prop;
                    for(Iterator it = l.iterator(); it.hasNext(); traitsList.add(prop))
                        prop = (String)it.next();

                }
                catch(NumberFormatException ex)
                {
                    throw new MessageException("Invalid traits reference: " + id);
                }
                catch(IndexOutOfBoundsException ex)
                {
                    throw new MessageException("Unknown traits reference: " + id);
                }
            } else
            {
                boolean externalizable = false;
                String ext = attributes.getValue("externalizable");
                if(ext != null)
                    externalizable = "true".equals(ext.trim());
                Object obj = objectStack.peek();
                if(externalizable && !(obj instanceof Externalizable))
                {
                    SerializationException ex = new SerializationException();
                    ex.setMessage(10305, new Object[] {
                        obj.getClass().getName()
                    });
                    throw ex;
                }
                traitsTable.add(traitsList);
            }
            isTraitProperty = true;
        } else
        {
            throw new MessageException("Unexpected traits");
        }
    }

    public void end_traits()
    {
        isTraitProperty = false;
    }

    public void start_true(Attributes attributes)
    {
        setValue(Boolean.TRUE);
        if(isDebug)
            trace.write(true);
    }

    public void end_true()
    {
    }

    public void start_undefined(Attributes attributes)
    {
        setValue(null);
        if(isDebug)
            trace.writeUndefined();
    }

    public void end_undefined()
    {
    }

    public void start_xml(Attributes attributes)
    {
        text.delete(0, text.length());
    }

    public void end_xml()
    {
        String xml = text.toString();
        xml = unescapeCloseCDATA(xml);
        Object value = xmlDeserializer.createObject(null, xml);
        setValue(value);
    }

    private String unescapeCloseCDATA(String s)
    {
        if(s.length() > 5 && s.indexOf("]]&gt;") != -1)
            s = s.replaceAll("]]&gt;", "]]>");
        return s;
    }

    private void setValue(Object value)
    {
        Object obj;
        Externalizable extern;
        Amf3Input objIn;
        ByteArrayInputStream baIn;
        if(objectStack.empty())
        {
            if(currentHeader != null)
                currentHeader.setData(value);
            else
            if(currentBody != null)
                currentBody.setData(value);
            else
                throw new MessageException("Unexpected value: " + value);
            return;
        }
        obj = objectStack.peek();
        if(!(obj instanceof Externalizable))
            break MISSING_BLOCK_LABEL_287;
        if(value == null || !value.getClass().isArray() || !Byte.TYPE.equals(value.getClass().getComponentType()))
            break MISSING_BLOCK_LABEL_277;
        extern = (Externalizable)obj;
        objIn = new Amf3Input(context);
        byte ba[] = (byte[])(byte[])value;
        baIn = new ByteArrayInputStream(ba);
        try
        {
            objIn.setInputStream(baIn);
            extern.readExternal(objIn);
        }
        catch(ClassNotFoundException ex)
        {
            throw new MessageException("Error while reading Externalizable class " + extern.getClass().getName(), ex);
        }
        catch(IOException ex)
        {
            throw new MessageException("Error while reading Externalizable class " + extern.getClass().getName(), ex);
        }
        try
        {
            objIn.close();
        }
        catch(IOException ex) { }
        break MISSING_BLOCK_LABEL_804;
        Exception exception;
        exception;
        try
        {
            objIn.close();
        }
        catch(IOException ex) { }
        throw exception;
        throw new MessageException("Error while reading Externalizable class. Value must be a byte array.");
        if(obj instanceof ASObject)
        {
            TraitsContext traitsContext = (TraitsContext)traitsStack.peek();
            String prop;
            try
            {
                prop = traitsContext.next();
            }
            catch(IndexOutOfBoundsException ex)
            {
                throw new MessageException("Object has no trait info for value: " + value);
            }
            ASObject aso = (ASObject)obj;
            aso.put(prop, value);
            if(isDebug)
                trace.namedElement(prop);
        } else
        if((obj instanceof ArrayList) && !(obj instanceof ArrayCollection))
        {
            ArrayList list = (ArrayList)obj;
            list.add(value);
            if(isDebug)
                trace.arrayElement(list.size() - 1);
        } else
        if(obj.getClass().isArray())
        {
            if(!strictArrayIndexStack.empty())
            {
                int indexObj[] = (int[])(int[])strictArrayIndexStack.peek();
                int index = indexObj[0];
                if(Array.getLength(obj) > index)
                    Array.set(obj, index, value);
                else
                    throw new MessageException("Index out of bounds at: " + index + " cannot set array value: " + value + "");
                indexObj[0]++;
            }
        } else
        if(obj instanceof Map)
        {
            Map map = (Map)obj;
            if(!arrayPropertyStack.empty())
            {
                String prop = (String)arrayPropertyStack.peek();
                map.put(prop, value);
                if(isDebug)
                    trace.namedElement(prop);
                return;
            }
            if(!ecmaArrayIndexStack.empty())
            {
                int index[] = (int[])(int[])ecmaArrayIndexStack.peek();
                String prop = String.valueOf(index[0]);
                index[0]++;
                map.put(prop, value);
                if(isDebug)
                    trace.namedElement(prop);
            }
        } else
        {
            TraitsContext traitsContext = (TraitsContext)traitsStack.peek();
            String prop;
            try
            {
                prop = traitsContext.next();
            }
            catch(IndexOutOfBoundsException ex)
            {
                throw new MessageException("Object has no trait info for value: " + value, ex);
            }
            try
            {
                PropertyProxy proxy = PropertyProxyRegistry.getProxyAndRegister(obj);
                proxy = (PropertyProxy)proxy.clone();
                proxy.setValue(obj, prop, value);
            }
            catch(Exception ex)
            {
                throw new MessageException("Failed to set property '" + prop + "' with value: " + value, ex);
            }
            if(isDebug)
                trace.namedElement(prop);
        }
    }

    private SerializationContext context;
    private BeanProxy beanproxy;
    private CustomDeserializer xmlDeserializer;
    private CustomDeserializer byteArrayDeserializer;
    private static final CustomDeserializer DEFAULT_XML_DESERIALIZER = new XMLDeserializer();
    private static final CustomDeserializer DEFAULT_BYTE_ARRAY_DESERIALIZER = new ByteArrayDeserializer();
    private final ArrayList objectTable = new ArrayList(64);
    private final ArrayList stringTable = new ArrayList(64);
    private final ArrayList traitsTable = new ArrayList(10);
    private StringBuffer text;
    private ActionMessage message;
    private MessageHeader currentHeader;
    private MessageBody currentBody;
    private Stack objectStack;
    private Stack arrayPropertyStack;
    private Stack ecmaArrayIndexStack;
    private Stack strictArrayIndexStack;
    private Stack traitsStack;
    private boolean isStringReference;
    private boolean isTraitProperty;
    protected boolean isDebug;
    protected AmfTrace trace;

}
