// Decompiled by Jad v1.5.8g. Copyright 2001 Pavel Kouznetsov.
// Jad home page: http://www.kpdus.com/jad.html
// Decompiler options: packimports(3) 
// Source File Name:   AmfxOutput.java

package flex.messaging.io.amfx;

import flex.messaging.io.*;
import flex.messaging.io.amf.*;
import flex.messaging.io.amf.serializers.CustomSerializer;
import flex.messaging.io.amfx.serializers.ByteArraySerializer;
import flex.messaging.util.Trace;
import java.io.*;
import java.lang.reflect.Array;
import java.util.*;
import javax.sql.RowSet;
import org.w3c.dom.Document;

// Referenced classes of package flex.messaging.io.amfx:
//            AmfxTypes

public class AmfxOutput extends AbstractAmfOutput
    implements AmfxTypes
{

    public AmfxOutput(SerializationContext context)
    {
        super(context);
        configDefaults();
        objectTable = new IdentityHashMap(64);
        traitsTable = new HashMap(10);
        stringTable = new HashMap(64);
    }

    private void configDefaults()
    {
        byteArraySerializer = context.getCustomSerializer("byte-array");
        if(byteArraySerializer == null)
            byteArraySerializer = DEFAULT_BYTE_ARRAY_SERIALIZER;
    }

    public void reset()
    {
        super.reset();
        objectTable.clear();
        traitsTable.clear();
        stringTable.clear();
    }

    public void writeObject(Object o)
        throws IOException
    {
        if(o == null)
        {
            writeAMFNull();
            return;
        }
        if((o instanceof String) || (o instanceof Character))
        {
            String s = o.toString();
            writeString(s);
        } else
        if(o instanceof Number)
        {
            if((o instanceof Integer) || (o instanceof Short) || (o instanceof Byte))
            {
                int i = ((Number)o).intValue();
                writeAMFInt(i);
            } else
            {
                double d = ((Number)o).doubleValue();
                writeAMFDouble(d);
            }
        } else
        if(o instanceof Boolean)
            writeAMFBoolean(((Boolean)o).booleanValue());
        else
        if(o instanceof Date)
            writeDate((Date)o);
        else
        if(o instanceof Calendar)
            writeDate(((Calendar)o).getTime());
        else
        if(o instanceof Document)
        {
            documentSerializer.writeObject(this, o);
        } else
        {
            Class cls = o.getClass();
            if((o instanceof Map) && context.legacyMap && !(o instanceof ASObject))
                writeMapAsECMAArray((Map)o);
            else
            if(o instanceof Collection)
            {
                if(context.legacyCollection)
                    writeCollection((Collection)o, null);
                else
                    writeArrayCollection((Collection)o, null);
            } else
            if(cls.isArray())
            {
                writeAMFArray(o, cls.getComponentType());
            } else
            {
                if(o instanceof RowSet)
                    o = new PagedRowSet((RowSet)o, 0x7fffffff, false);
                else
                if((o instanceof Throwable) && context.legacyThrowable)
                    o = new StatusInfoProxy((Throwable)o);
                writeCustomObject(o);
            }
        }
    }

    public void writeObjectTraits(TraitsInfo ti)
        throws IOException
    {
        String className = ti.getClassName();
        if(className == null || className.length() == 0)
        {
            writeUTF("<object>");
        } else
        {
            int len = 127;
            StringBuffer sb = new StringBuffer(len);
            sb.append("<").append("object").append(" type=\"");
            sb.append(className);
            sb.append("\">");
            writeUTF(sb);
        }
        if(isDebug)
            trace.startAMFObject(className, objectTable.size() - 1);
        if(ti.length() == 0 && className == null)
            writeUTF("<traits/>");
        else
        if(!byReference(ti))
            if(ti.isExternalizable())
            {
                writeUTF("<traits externalizable=\"true\" />");
            } else
            {
                int count = ti.getProperties().size();
                if(count <= 0)
                {
                    writeUTF("<traits/>");
                } else
                {
                    writeUTF("<traits>");
                    for(int i = 0; i < count; i++)
                    {
                        String propName = ti.getProperty(i);
                        writeString(propName, true);
                    }

                    writeUTF("</traits>");
                }
            }
    }

    public void writeObjectProperty(String name, Object value)
        throws IOException
    {
        if(isDebug)
            trace.namedElement(name);
        writeObject(value);
    }

    public void writeObjectEnd()
        throws IOException
    {
        writeUTF("</object>");
        if(isDebug)
            trace.endAMFObject();
    }

    public void writeUTF(String s)
        throws IOException
    {
        byte bytes[] = s.getBytes("UTF-8");
        out.write(bytes);
    }

    private void writeAMFBoolean(boolean b)
        throws IOException
    {
        if(b)
            writeUTF("<true/>");
        else
            writeUTF("<false/>");
        if(isDebug)
            trace.write(b);
    }

    private void writeAMFDouble(double d)
        throws IOException
    {
        int buflen = 40;
        StringBuffer sb = new StringBuffer(buflen);
        sb.append("<double>");
        sb.append(d);
        sb.append("</double>");
        writeUTF(sb);
        if(isDebug)
            trace.write(d);
    }

    private void writeAMFInt(int i)
        throws IOException
    {
        int buflen = 25;
        StringBuffer sb = new StringBuffer(buflen);
        sb.append("<int>");
        sb.append(i);
        sb.append("</int>");
        writeUTF(sb);
        if(isDebug)
            trace.write(i);
    }

    private void writeByteArray(byte ba[])
        throws IOException
    {
        int length = ba.length * 2;
        int len = 23 + length;
        StringBuffer sb = new StringBuffer(len);
        sb.append("<bytearray>");
        writeUTF(sb);
        byteArraySerializer.writeObject(this, ba);
        writeUTF("</bytearray>");
        if(isDebug)
            trace.startByteArray(objectTable.size() - 1, ba.length);
    }

    private void writeByteArray(Byte ba[])
        throws IOException
    {
        int length = ba.length;
        byte bytes[] = new byte[length];
        for(int i = 0; i < length; i++)
        {
            Byte b = ba[i];
            if(b == null)
                bytes[i] = 0;
            else
                bytes[i] = ba[i].byteValue();
        }

        writeByteArray(bytes);
    }

    public void writeUTF(StringBuffer sb)
        throws IOException
    {
        byte bytes[] = sb.toString().getBytes("UTF-8");
        out.write(bytes);
    }

    private void writeDate(Date d)
        throws IOException
    {
        if(!byReference(d))
        {
            int buflen = 30;
            long time = d.getTime();
            StringBuffer sb = new StringBuffer(buflen);
            sb.append("<date>");
            sb.append(time);
            sb.append("</date>");
            writeUTF(sb);
            if(isDebug)
                trace.write(d);
        }
    }

    private void writeMapAsECMAArray(Map map)
        throws IOException
    {
        int len = 20;
        StringBuffer sb = new StringBuffer(len);
        sb.append("<").append("array").append(" ecma=\"true\">");
        writeUTF(sb);
        if(isDebug)
            trace.startAMFArray(objectTable.size() - 1);
        Iterator it = map.keySet().iterator();
        do
        {
            if(!it.hasNext())
                break;
            Object key = it.next();
            if(key != null)
            {
                String propName = key.toString();
                sb = new StringBuffer();
                sb.append("<").append("item").append(" name=\"").append(propName).append("\">");
                writeUTF(sb);
                if(isDebug)
                    trace.namedElement(propName);
                writeObject(map.get(key));
                writeUTF("</item>");
            }
        } while(true);
        writeUTF("</array>");
        if(isDebug)
            trace.endAMFArray();
    }

    private void writeAMFNull()
        throws IOException
    {
        writeUTF("<null/>");
        if(isDebug)
            trace.writeNull();
    }

    private void writeString(String s)
        throws IOException
    {
        writeString(s, false);
        if(isDebug)
            trace.writeString(s);
    }

    public void writeStringAsXML(String str)
        throws IOException
    {
        int len = str.length() + 15;
        StringBuffer sb = new StringBuffer(len);
        sb.append("<xml>");
        writeEscapedString(sb, str);
        sb.append("</xml>");
        writeUTF(sb);
        if(isDebug)
            trace.writeString(str);
    }

    private void writeAMFArray(Object o, Class componentType)
        throws IOException
    {
        if(componentType.isPrimitive())
            writePrimitiveArray(o);
        else
        if(componentType.equals(java.lang.Byte.class))
            writeByteArray((Byte[])(Byte[])o);
        else
        if(componentType.equals(java.lang.Character.class))
            writeCharArrayAsString((Character[])(Character[])o);
        else
            writeObjectArray((Object[])(Object[])o, null);
    }

    private void writeArrayCollection(Collection col, SerializationDescriptor desc)
        throws IOException
    {
        if(!byReference(col))
        {
            ArrayCollection ac;
            if(col instanceof ArrayCollection)
            {
                ac = (ArrayCollection)col;
            } else
            {
                ac = new ArrayCollection(col);
                if(desc != null)
                    ac.setDescriptor(desc);
            }
            PropertyProxy proxy = PropertyProxyRegistry.getProxy(ac);
            writePropertyProxy(proxy, ac);
        }
    }

    private void writeCustomObject(Object o)
        throws IOException
    {
        PropertyProxy proxy = null;
        if(o instanceof PropertyProxy)
        {
            proxy = (PropertyProxy)o;
            o = proxy.getDefaultInstance();
            if(o == null)
            {
                writeAMFNull();
                return;
            }
            if(o instanceof Collection)
            {
                if(context.legacyCollection)
                    writeCollection((Collection)o, proxy.getDescriptor());
                else
                    writeArrayCollection((Collection)o, proxy.getDescriptor());
                return;
            }
            if(o.getClass().isArray())
            {
                writeObjectArray((Object[])(Object[])o, proxy.getDescriptor());
                return;
            }
            if(context.legacyMap && (o instanceof Map) && !(o instanceof ASObject))
            {
                writeMapAsECMAArray((Map)o);
                return;
            }
        }
        if(!byReference(o))
        {
            if(proxy == null)
                proxy = PropertyProxyRegistry.getProxyAndRegister(o);
            writePropertyProxy(proxy, o);
        }
    }

    private void writePropertyProxy(PropertyProxy pp, Object instance)
        throws IOException
    {
        List propertyNames = null;
        boolean externalizable = pp.isExternalizable(instance);
        if(!externalizable)
            propertyNames = pp.getPropertyNames(instance);
        TraitsInfo ti = new TraitsInfo(pp.getAlias(instance), pp.isDynamic(), externalizable, propertyNames);
        writeObjectTraits(ti);
        if(externalizable)
        {
            ByteArrayOutputStream bout = new ByteArrayOutputStream();
            Amf3Output objOut = new Amf3Output(context);
            objOut.setOutputStream(bout);
            ((Externalizable)instance).writeExternal(objOut);
            writeByteArray(bout.toByteArray());
        } else
        if(propertyNames != null)
        {
            String propName;
            Object value;
            for(Iterator it = propertyNames.iterator(); it.hasNext(); writeObjectProperty(propName, value))
            {
                propName = (String)it.next();
                value = pp.getValue(instance, propName);
            }

        }
        writeObjectEnd();
    }

    private void writeString(String s, boolean isTrait)
        throws IOException
    {
        if(s.length() == 0)
            writeUTF("<string/>");
        else
        if(!byReference(s))
        {
            int len = s.length() + 35;
            StringBuffer sb = new StringBuffer(len);
            sb.append("<string>");
            if(!isTrait)
                writeEscapedString(sb, s);
            else
                sb.append(s);
            sb.append("</string>");
            writeUTF(sb);
        }
    }

    private void writeEscapedString(StringBuffer sb, String s)
    {
        StringBuffer temp = new StringBuffer(s.length());
        char chars[] = s.toCharArray();
        for(int i = 0; i < chars.length; i++)
        {
            char c = chars[i];
            if(c >= ' ')
            {
                if(c == '&')
                {
                    temp.append("&amp;");
                    continue;
                }
                if(c == '<')
                {
                    temp.append("&lt;");
                    continue;
                }
                if(c > '\uD7FF' && (c < '\uE000' || c > '\uFFFD'))
                    temp.append("&#x").append(Integer.toHexString(c)).append(";");
                else
                    temp.append(c);
                continue;
            }
            if(c == '\t' || c == '\n' || c == '\r')
                temp.append(c);
            else
                temp.append("&#x").append(Integer.toHexString(c)).append(";");
        }

        sb.append(temp);
    }

    private void writeCharArrayAsString(Character ca[])
        throws IOException
    {
        int length = ca.length;
        char chars[] = new char[length];
        for(int i = 0; i < length; i++)
        {
            Character c = ca[i];
            if(c == null)
                chars[i] = '\0';
            else
                chars[i] = ca[i].charValue();
        }

        writeCharArrayAsString(chars);
    }

    private void writeCharArrayAsString(char ca[])
        throws IOException
    {
        String str = new String(ca);
        writeString(str);
    }

    private void writeCollection(Collection c, SerializationDescriptor descriptor)
        throws IOException
    {
        if(!byReference(c))
            writeObjectArrayDirectly(c.toArray(), descriptor);
    }

    private void writeObjectArray(Object values[], SerializationDescriptor descriptor)
        throws IOException
    {
        if(!byReference(((Object) (values))))
            writeObjectArrayDirectly(values, descriptor);
    }

    private void writeObjectArrayDirectly(Object values[], SerializationDescriptor descriptor)
        throws IOException
    {
        int len = 25;
        StringBuffer sb = new StringBuffer(len);
        sb.append("<").append("array").append(" length=\"");
        sb.append(values.length);
        sb.append("\">");
        writeUTF(sb);
        if(isDebug)
            trace.startAMFArray(objectTable.size() - 1);
        for(int i = 0; i < values.length; i++)
        {
            if(isDebug)
                trace.arrayElement(i);
            writeObject(values[i]);
        }

        writeUTF("</array>");
        if(isDebug)
            trace.endAMFArray();
    }

    private void writePrimitiveArray(Object obj)
        throws IOException
    {
        Class aType = obj.getClass().getComponentType();
        if(aType.equals(Character.TYPE))
        {
            char c[] = (char[])(char[])obj;
            writeCharArrayAsString(c);
        } else
        if(aType.equals(Byte.TYPE))
            writeByteArray((byte[])(byte[])obj);
        else
        if(!byReference(obj))
        {
            int length = Array.getLength(obj);
            int buflen = 25;
            StringBuffer sb = new StringBuffer(buflen);
            sb.append("<").append("array").append(" length=\"");
            sb.append(length);
            sb.append("\">");
            writeUTF(sb);
            if(isDebug)
                trace.startAMFArray(objectTable.size() - 1);
            if(aType.equals(Boolean.TYPE))
            {
                boolean b[] = (boolean[])(boolean[])obj;
                for(int i = 0; i < b.length; i++)
                {
                    if(isDebug)
                        trace.arrayElement(i);
                    writeAMFBoolean(b[i]);
                }

            } else
            if(aType.equals(Integer.TYPE) || aType.equals(Short.TYPE))
            {
                for(int i = 0; i < length; i++)
                {
                    if(isDebug)
                        trace.arrayElement(i);
                    int v = Array.getInt(obj, i);
                    writeAMFInt(v);
                }

            } else
            {
                for(int i = 0; i < length; i++)
                {
                    if(isDebug)
                        trace.arrayElement(i);
                    double v = Array.getDouble(obj, i);
                    writeAMFDouble(v);
                }

            }
            writeUTF("</array>");
            if(isDebug)
                trace.endAMFArray();
        }
    }

    private boolean byReference(Object o)
        throws IOException
    {
        Object ref = objectTable.get(o);
        if(ref != null)
            try
            {
                int refNum = ((Integer)ref).intValue();
                int len = 20;
                StringBuffer sb = new StringBuffer(len);
                sb.append("<").append("ref").append(" id=\"");
                sb.append(refNum);
                sb.append("\"/>");
                writeUTF(sb);
                if(isDebug)
                    trace.writeRef(refNum);
            }
            catch(ClassCastException e)
            {
                throw new IOException("Object reference is not an Integer");
            }
        else
            objectTable.put(o, new Integer(objectTable.size()));
        return ref != null;
    }

    private boolean byReference(String s)
        throws IOException
    {
        Object ref = stringTable.get(s);
        if(ref != null)
            try
            {
                int refNum = ((Integer)ref).intValue();
                int len = 20;
                StringBuffer sb = new StringBuffer(len);
                sb.append("<").append("string").append(" id=\"");
                sb.append(refNum);
                sb.append("\"/>");
                writeUTF(sb);
                if(Trace.amf && isDebug)
                    trace.writeStringRef(refNum);
            }
            catch(ClassCastException e)
            {
                throw new IOException("String reference is not an Integer");
            }
        else
            stringTable.put(s, new Integer(stringTable.size()));
        return ref != null;
    }

    private boolean byReference(TraitsInfo ti)
        throws IOException
    {
        if(ti.length() == 0 && ti.getClassName() == null)
            return false;
        Object ref = traitsTable.get(ti);
        if(ref != null)
            try
            {
                int refNum = ((Integer)ref).intValue();
                int len = 20;
                StringBuffer sb = new StringBuffer(len);
                sb.append("<").append("traits").append(" id=\"");
                sb.append(refNum);
                sb.append("\"/>");
                writeUTF(sb);
                if(Trace.amf && isDebug)
                    trace.writeTraitsInfoRef(refNum);
            }
            catch(ClassCastException e)
            {
                throw new IOException("Traits reference is not an Integer");
            }
        else
            traitsTable.put(ti, new Integer(traitsTable.size()));
        return ref != null;
    }

    private IdentityHashMap objectTable;
    private HashMap traitsTable;
    private HashMap stringTable;
    private CustomSerializer byteArraySerializer;
    private static final CustomSerializer DEFAULT_BYTE_ARRAY_SERIALIZER = new ByteArraySerializer();

}
