// Decompiled by Jad v1.5.8g. Copyright 2001 Pavel Kouznetsov.
// Jad home page: http://www.kpdus.com/jad.html
// Decompiler options: packimports(3) 
// Source File Name:   AbstractConfigurationParser.java

package flex.messaging.config;

import flex.messaging.util.FileUtils;
import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.w3c.dom.*;
import org.xml.sax.*;

// Referenced classes of package flex.messaging.config:
//            ConfigurationException, ConfigMap, ConfigurationParser, ConfigurationConstants, 
//            ConfigurationFileResolver, ServicesConfiguration

public abstract class AbstractConfigurationParser
    implements ConfigurationParser, ConfigurationConstants
{

    protected AbstractConfigurationParser()
    {
        fileByDocument = new HashMap();
        initializeDocumentBuilder();
    }

    public void parse(String path, ConfigurationFileResolver fileResolver, ServicesConfiguration config)
    {
        this.config = config;
        this.fileResolver = fileResolver;
        Document doc = loadDocument(path, fileResolver.getConfigurationFile(path));
        initializeExpressionQuery();
        parseTopLevelConfig(doc);
    }

    protected void initializeDocumentBuilder()
    {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setIgnoringComments(true);
        factory.setIgnoringElementContentWhitespace(true);
        try
        {
            docBuilder = factory.newDocumentBuilder();
        }
        catch(ParserConfigurationException ex)
        {
            ConfigurationException e = new ConfigurationException();
            e.setMessage(10100);
            e.setRootCause(ex);
            throw e;
        }
    }

    protected Document loadDocument(String path, InputStream in)
    {
        try
        {
            if(!in.markSupported())
                in = new BufferedInputStream(in);
            String encoding = FileUtils.consumeBOM(in, null);
            Document doc;
            if("UTF-8".equals(encoding) || "UTF-16".equals(encoding))
            {
                InputSource inputSource = new InputSource(in);
                inputSource.setEncoding(encoding);
                doc = docBuilder.parse(inputSource);
            } else
            {
                doc = docBuilder.parse(in);
            }
            addFileByDocument(path, doc);
            doc.getDocumentElement().normalize();
            return doc;
        }
        catch(SAXParseException ex)
        {
            Integer line = new Integer(ex.getLineNumber());
            Integer col = new Integer(ex.getColumnNumber());
            String message = ex.getMessage();
            ConfigurationException e = new ConfigurationException();
            e.setMessage(10102, new Object[] {
                line, col, message
            });
            e.setRootCause(ex);
            throw e;
        }
        catch(SAXException ex)
        {
            ConfigurationException e = new ConfigurationException();
            e.setMessage(ex.getMessage());
            e.setRootCause(ex);
            throw e;
        }
        catch(IOException ex)
        {
            ConfigurationException e = new ConfigurationException();
            e.setMessage(ex.getMessage());
            e.setRootCause(ex);
            throw e;
        }
    }

    protected void addFileByDocument(String path, Node node)
    {
        String shortPath = path;
        if(shortPath != null && (shortPath.indexOf("/") != -1 || shortPath.indexOf("\\") != -1))
        {
            int start = 0;
            start = shortPath.lastIndexOf("/");
            if(start == -1)
                start = shortPath.lastIndexOf("\\");
            shortPath = path.substring(start + 1);
        }
        fileByDocument.put(node, shortPath);
    }

    protected String getSourceFileOf(Node node)
    {
        return (String)fileByDocument.get(node.getOwnerDocument());
    }

    protected abstract void parseTopLevelConfig(Document document);

    protected abstract void initializeExpressionQuery();

    protected abstract Node selectSingleNode(Node node, String s);

    protected abstract NodeList selectNodeList(Node node, String s);

    protected abstract Object evaluateExpression(Node node, String s);

    public ConfigMap properties(NodeList properties)
    {
        int length = properties.getLength();
        ConfigMap map = new ConfigMap(length);
        for(int p = 0; p < length; p++)
        {
            Node prop = properties.item(p);
            String propName = prop.getNodeName();
            if(propName == null)
                continue;
            propName = propName.trim();
            if(prop.getNodeType() == 1)
            {
                NodeList attributes = selectNodeList(prop, "@*");
                NodeList children = selectNodeList(prop, "*");
                if(children.getLength() > 0 || attributes.getLength() > 0)
                {
                    ConfigMap childMap = new ConfigMap();
                    if(children.getLength() > 0)
                        childMap.addProperties(properties(children));
                    if(attributes.getLength() > 0)
                        childMap.addProperties(properties(attributes));
                    map.addProperty(propName, childMap);
                } else
                {
                    String propValue = evaluateExpression(prop, ".").toString();
                    map.addProperty(propName, propValue);
                }
            } else
            {
                map.addProperty(propName, prop.getNodeValue());
            }
        }

        return map;
    }

    public String getAttributeOrChildElement(Node node, String name)
    {
        String attr = evaluateExpression(node, "@" + name).toString().trim();
        if(attr.length() == 0)
            attr = evaluateExpression(node, name).toString().trim();
        return attr;
    }

    public void allowedChildElements(Node node, String allowed[])
    {
        NodeList children = selectNodeList(node, "*");
        String unexpected = unexpected(children, allowed);
        if(unexpected != null)
        {
            ConfigurationException ex = new ConfigurationException();
            Object args[] = {
                unexpected, node.getNodeName(), getSourceFilename(node)
            };
            ex.setMessage(10106, args);
            throw ex;
        }
        NodeList textNodes = selectNodeList(node, "text()");
        for(int i = 0; i < textNodes.getLength(); i++)
        {
            String text = evaluateExpression(textNodes.item(i), ".").toString().trim();
            if(text.length() > 0)
            {
                ConfigurationException ex = new ConfigurationException();
                Object args[] = {
                    text, node.getNodeName(), getSourceFilename(node)
                };
                ex.setMessage(11104, args);
                throw ex;
            }
        }

    }

    public void allowedAttributes(Node node, String allowed[])
    {
        NodeList attributes = selectNodeList(node, "@*");
        String unexpectedAttribute = unexpected(attributes, allowed);
        if(unexpectedAttribute != null)
        {
            ConfigurationException ex = new ConfigurationException();
            Object args[] = {
                unexpectedAttribute, node.getNodeName(), getSourceFilename(node)
            };
            ex.setMessage(10107, args);
            throw ex;
        } else
        {
            return;
        }
    }

    private String getSourceFilename(Node node)
    {
        return getSourceFileOf(node);
    }

    public void allowedAttributesOrElements(Node node, String allowed[])
    {
        allowedAttributes(node, allowed);
        allowedChildElements(node, allowed);
    }

    public void requiredAttributesOrElements(Node node, String required[])
    {
        String nodeName = node.getNodeName();
        NodeList attributes = selectNodeList(node, "@*");
        List list = new ArrayList();
        for(int i = 0; i < required.length; i++)
            list.add(required[i]);

        String missingAttribute = null;
        do
        {
            missingAttribute = required(attributes, list);
            if(missingAttribute != null)
            {
                Node child = selectSingleNode(node, missingAttribute);
                if(child != null)
                {
                    list.remove(missingAttribute);
                } else
                {
                    ConfigurationException ex = new ConfigurationException();
                    ex.setMessage(10105, new Object[] {
                        missingAttribute, nodeName
                    });
                    throw ex;
                }
            }
        } while(missingAttribute != null && list.size() > 0);
    }

    public void requiredChildElements(Node node, String required[])
    {
        String nodeName = node.getNodeName();
        NodeList children = selectNodeList(node, "*");
        List list = new ArrayList();
        for(int i = 0; i < required.length; i++)
            list.add(required[i]);

        String missing = required(children, list);
        if(missing != null)
        {
            ConfigurationException ex = new ConfigurationException();
            ex.setMessage(10104, new Object[] {
                missing, nodeName
            });
            throw ex;
        } else
        {
            return;
        }
    }

    public static String unexpected(NodeList attributes, String allowed[])
    {
        for(int i = 0; i < attributes.getLength(); i++)
        {
            Node attrib = attributes.item(i);
            String attribName = attrib.getNodeName();
            boolean match = false;
            int j = 0;
            do
            {
                if(j >= allowed.length)
                    break;
                String a = allowed[j];
                if(a.equals(attribName))
                {
                    match = true;
                    break;
                }
                j++;
            } while(true);
            if(!match)
                return attribName;
        }

        return null;
    }

    public static String required(NodeList attributes, List required)
    {
        for(int i = 0; i < required.size(); i++)
        {
            boolean found = false;
            String req = (String)required.get(i);
            int j = 0;
            do
            {
                if(j >= attributes.getLength())
                    break;
                Node attrib = attributes.item(j);
                String attribName = attrib.getNodeName();
                if(req.equals(attribName))
                {
                    found = true;
                    break;
                }
                j++;
            } while(true);
            if(!found)
                return req;
        }

        return null;
    }

    public static boolean isValidID(String id)
    {
        if(id != null && id.length() > 0 && id.length() < 256)
        {
            char chars[] = id.toCharArray();
            for(int i = 0; i < chars.length; i++)
            {
                char c = chars[i];
                if(",;:".indexOf(c) != -1)
                    return false;
            }

            return true;
        } else
        {
            return false;
        }
    }

    protected ServicesConfiguration config;
    protected DocumentBuilder docBuilder;
    protected ConfigurationFileResolver fileResolver;
    private Map fileByDocument;
}
